/*
 * (C) Copyright Pattern Recognition Group Delft. All rights reserved.
 * 
 * ESTIMAT.C : Estimate mean and variance for a given set of random samples.
 * 
 * Made by : W.F. Schmidt 
 * SccsId  : %Z%$RCSfile: estimat.c,v $ V%Q% $Revision: 2.4 $ $Date: 1996/10/07 06:50:24 $
 */

#include <stdio.h>
#include <math.h>

#define ESTIMAT
#include <sprlib.h>

static long points = 0;
static double **cov = NULL;
static double *mean = NULL;

/*
 * MALLOC_ESTIMATE : Allocation and initialization of memory to hold
 * summations needed to estimate covariance matrices and mean values of
 * multidimensional patterns. Parameter n defines the dimensionality of the
 * patterns.
 *
 * Return value : TRUE If an error is detected.
 *                FALSE If no error is detected.
 */

static int malloc_estimate (int n)
{
   int i, j;

   points = 0L;
   if ((cov = matrix(1, n, 1, n)) == NULL)
		return TRUE;

   if ((mean = vector(1, n)) == NULL)
		return TRUE;

   for (i = 1; i <= n; i++)
   {
      mean[i] = 0.;
      for (j = 1; j <= n; j++)
         cov[i][j] = 0.;
   }

	return FALSE;
}

/*
 * FREE_ESTIMATE : Free the memory used to calculate the summmations.
 */

static void free_estimate (int n)
{
   if ((cov==NULL) || (mean==NULL))
      return;

   free_matrix(cov, 1, n, 1, n);
   free_vector(mean, 1, n);

   points = 0L;
   cov = NULL;
   mean = NULL;
}

/*
 * ADD_MEAN_VAR : Add a pattern to the summations, needed to calculate mean
 * and covariance matrix of a multidimensional stochastic process. The para-
 * meter n defines the dimension of the pattern. Pattern is assumed to be a
 * vector pattern[1..n] in NR format.
 */

int add_mean_var (double *pattern, int n)
{
   int i, j;

   if (n < 1)
	{
      sprerror("add_mean_var(): Illegal dimension specified.");
		return TRUE;
	}

   if (cov == NULL || mean == NULL || points == 0L)
	{
      if (malloc_estimate(n))
			return TRUE;
	}

   points++;
   for (i = 1; i <= n; i++)
   {
      mean[i] += pattern[i];
      for (j = i; j <= n; j++)
         cov[i][j] += pattern[i] * pattern[j];
   }

	return FALSE;
}

/*
 * CALC_MEAN_VAR : Calculate the covariance matrix and the mean values of a
 * multidimensional stochastic pattern. It is assumed that the patterns are
 * supplied to this routine with the ADD_ESTIMATE function. The supplied
 * parameters are n (=dimension), m[1..n] the mean vector to be calculated,
 * c[1..n][1..n] the covariance matrix. The vector m and matrix c must be in
 * NR format.
 */                                                    

int calc_mean_var (int n, double *m, double **c)
{
   int i, j;

   if (cov == NULL || mean == NULL || points == 0L)
	{
		sprerror("calc_mean_var(): No data found.");
		return TRUE;
	}

   if (points < 2L)
	{
      sprerror("calc_mean_var(): Number of points must be >= 2.");
		return TRUE;
	}

   for (i = 1; i <= n; i++)
      m[i] = mean[i] /= (double) points;

   for (i = 1; i <= n; i++)
   {
      for (j = i; j <= n; j++)
      {
         c[i][j] = c[j][i] = (cov[i][j] - ((double) points) * m[i] * m[j]);
         c[i][j] = c[j][i] /= ((double) (points - 1L));
      }
   }
   free_estimate(n);

	return FALSE;
}

#undef ESTIMAT

/*
 * End of Source
 */
